@@ -22,48 +22,52 @@ use std::fmt::Debug;
 use std::rc::Rc;
 
 /// Indicates that an iterator has passed beyond the limits of the list.
-pub const OUT_OF_BOUNDS:usize = usize::MAX;
+pub const OUT_OF_BOUNDS: usize = usize::MAX;
 
 #[derive(thiserror::Error, Debug)]
 pub enum MapError {
     #[error("error: Some error with the linked list")]
     InternalError(String),
+    #[error(transparent)]
+    BorrowError(#[from] std::cell::BorrowError),
+    #[error(transparent)]
+    BorrowMutError(#[from] std::cell::BorrowMutError),
 }
 
 #[cfg(test)]
 mod test;
 
 #[derive(Clone, Debug)]
-struct Node<T, U>
-    where
-        T: Clone + Debug,
-        U: Clone + Debug,
+struct Node<K, V>
+where
+    K: Debug,
+    V: Debug,
 {
     prev_: usize,
     next_: usize,
-    key_: T,
-    value_: U,
+    key_: K,
+    value_: V,
 }
 
 /// A double linked min list.
 /// The head (top/front) of the list is the first item. Sorted Order::Less than other items.
 /// The tail (bottom/back) is the last item of the list. Sorted Order::Greater than other items.
 #[derive(Clone, Debug)]
-pub struct LinkedList<T, U>
-    where
-        T: Clone + Debug,
-        U: Clone + Debug,
+pub struct LinkedList<K, V>
+where
+    K: Debug,
+    V: Debug,
 {
     head_: usize,
     tail_: usize,
-    nodes_: Vec<Option<Node<T, U>>>,
+    nodes_: Vec<Option<Node<K, V>>>,
     id_pool_: Vec<usize>,
 }
 
-impl<T, U> Default for LinkedList<T, U>
-    where
-        T: Clone + Debug,
-        U: Clone + Debug,
+impl<K, V> Default for LinkedList<K, V>
+where
+    K: Debug,
+    V: Debug,
 {
     fn default() -> Self {
         Self {
@@ -87,12 +91,24 @@ struct EraseOperation {
 }
 
 #[allow(dead_code)]
-impl<'a, T: 'a, U: 'a> LinkedList<T, U>
-    where
-        T: Clone + Debug + Ord + PartialOrd,
-        U: Clone + Debug,
+impl<'a, K: 'a, V: 'a> LinkedList<K, V>
+where
+    K: Debug + Ord + PartialOrd,
+    V: Debug,
 {
-    pub fn iter(&self) -> ListIterator<'_, T, U> {
+    /// Constructs a new, empty LinkedList<K,V> with the specified capacity.
+    /// The LinkedList will be able to hold exactly capacity elements without reallocating.
+    /// If capacity is 0, the list will not allocate.
+    pub fn with_capacity(capacity: usize) -> Self {
+        Self {
+            head_: OUT_OF_BOUNDS,
+            tail_: OUT_OF_BOUNDS,
+            nodes_: Vec::with_capacity(capacity),
+            id_pool_: Vec::with_capacity(capacity),
+        }
+    }
+
+    pub fn iter(&self) -> ListIterator<'_, K, V> {
         ListIterator {
             list_: self,
             my_next_: self.head_,
@@ -138,7 +154,7 @@ impl<'a, T: 'a, U: 'a> LinkedList<T, U>
 
     #[inline(always)]
     /// Returns the item key at index
-    pub fn get_k(&self, index: usize) -> Result<&T, MapError> {
+    pub fn get_k(&self, index: usize) -> Result<&K, MapError> {
         let rv = self
             .nodes_
             .get(index)
@@ -150,7 +166,7 @@ impl<'a, T: 'a, U: 'a> LinkedList<T, U>
 
     #[inline(always)]
     /// Returns the item value at index
-    pub fn get_v(&self, index: usize) -> Result<&U, MapError> {
+    pub fn get_v(&self, index: usize) -> Result<&V, MapError> {
         let rv = self
             .nodes_
             .get(index)
@@ -162,7 +178,14 @@ impl<'a, T: 'a, U: 'a> LinkedList<T, U>
 
     #[inline(always)]
     /// Returns the item key and value at index
-    pub fn get_kv(&self, index: usize) -> Result<(&T, &U), MapError> {
+    pub fn get(&self, index: usize) -> Result<(&K, &V), MapError> {
+        if index == OUT_OF_BOUNDS {
+            return Err(MapError::InternalError(format!(
+                "Invalid pointer (moved past start/end). {}:{}",
+                file!(),
+                line!()
+            )));
+        }
         let rv = self
             .nodes_
             .get(index)
@@ -174,7 +197,7 @@ impl<'a, T: 'a, U: 'a> LinkedList<T, U>
 
     #[inline(always)]
     /// Returns the previous key item of item at index
-    pub fn get_prev_k(&self, index: usize) -> Result<&T, MapError> {
+    pub fn get_prev_k(&self, index: usize) -> Result<&K, MapError> {
         let prev = self
             .nodes_
             .get(index)
@@ -196,7 +219,7 @@ impl<'a, T: 'a, U: 'a> LinkedList<T, U>
 
     /// Add an item at the front of the list
     /// Note that this ignores the order of items, use with care.
-    fn push_front_(&mut self, key: T, value: U) -> Result<usize, MapError> {
+    fn push_front_(&mut self, key: K, value: V) -> Result<usize, MapError> {
         let insertion_index = if !self.id_pool_.is_empty() {
             self.id_pool_.pop().unwrap()
         } else {
@@ -241,7 +264,7 @@ impl<'a, T: 'a, U: 'a> LinkedList<T, U>
     #[inline(always)]
     /// insert at position or append at back of the list
     /// Note that this ignores the order of items, use with care.
-    fn replace_or_push_(&mut self, insertion_index: usize, new_node: Node<T, U>) -> usize {
+    fn replace_or_push_(&mut self, insertion_index: usize, new_node: Node<K, V>) -> usize {
         if insertion_index == self.nodes_.len() {
             self.nodes_.push(Some(new_node));
         } else {
@@ -257,7 +280,7 @@ impl<'a, T: 'a, U: 'a> LinkedList<T, U>
 
     /// insert a new value before the element at index
     /// Note that this ignores the order of items, use with care.
-    fn insert_before_(&mut self, index: usize, key: T, value: U) -> Result<usize, MapError> {
+    fn insert_before_(&mut self, index: usize, key: K, value: V) -> Result<usize, MapError> {
         if index == OUT_OF_BOUNDS {
             return self.push_front_(key, value);
         }
@@ -336,7 +359,7 @@ impl<'a, T: 'a, U: 'a> LinkedList<T, U>
     }
 
     /// Add an item at the back of the list
-    fn push_back_(&mut self, key: T, value: U) -> Result<usize, MapError> {
+    fn push_back_(&mut self, key: K, value: V) -> Result<usize, MapError> {
         let insertion_index = if !self.id_pool_.is_empty() {
             self.id_pool_.pop().unwrap()
         } else {
@@ -385,28 +408,17 @@ impl<'a, T: 'a, U: 'a> LinkedList<T, U>
 
     #[inline(always)]
     /// Insert item at position defined by Order (lesser first)
-    /// 'before_equals' defines how new equal objects should be positioned:
-    /// true->closer to head, false->closer to tail
     /// This is the same as 'ordered_insert_pos()' with self.head_ as position hint
-    pub fn ordered_insert(
-        &mut self,
-        key: T,
-        value: U,
-        before_equals: bool,
-    ) -> Result<usize, MapError> {
-        self.ordered_insert_pos(key, value, self.head_, before_equals)
+    pub fn ordered_insert(&mut self, key: K, value: V) -> Result<usize, MapError> {
+        self.ordered_insert_pos(key, value, self.head_)
     }
 
     /// Insert item by Order (lesser first) with a position hint.
-    /// 'before_equals' defines how new equal objects should be positioned:
-    /// true-> before equals, false-> at tail end
-    /// todo: remove before_equals, always stick to tail end
     pub fn ordered_insert_pos(
         &mut self,
-        key: T,
-        value: U,
+        key: K,
+        value: V,
         position: usize,
-        before_equals: bool,
     ) -> Result<usize, MapError> {
         if self.head_ == OUT_OF_BOUNDS {
             // list is empty, ignore position and insert
@@ -424,7 +436,11 @@ impl<'a, T: 'a, U: 'a> LinkedList<T, U>
                     .unwrap()
                     .as_ref()
                     .ok_or_else(|| {
-                        MapError::InternalError(format!("head_ item was None {}:{}", file!(), line!()))
+                        MapError::InternalError(format!(
+                            "head_ item was None {}:{}",
+                            file!(),
+                            line!()
+                        ))
                     })?,
             ),
         };
@@ -433,62 +449,44 @@ impl<'a, T: 'a, U: 'a> LinkedList<T, U>
         //println!("curr_index:{}, first_node.key={:?}, cmp={:?}", curr_index, first_node.key, cmp);
 
         #[allow(clippy::collapsible_else_if)] // false positive?
-        if (cmp == Ordering::Greater) || ((cmp == Ordering::Equal) && !before_equals) {
-            if before_equals {
-                //println!("search down, insert before equals");
-                // we are searching down the list, stop at first Equal
-                while let Some(Some(sample)) = self.nodes_.get(curr_index) {
-                    // stop at first Ordering::Equal or Ordering::Less
-                    if key.cmp(&sample.key_) != Ordering::Greater {
-                        insert_before = Some(curr_index);
-                        break;
-                    } else {
-                        curr_index = sample.next_;
+        if (cmp == Ordering::Greater) || (cmp == Ordering::Equal) {
+            //println!("search down, insert after equals");
+            // we are searching down the list, stop at first Less
+            while let Some(Some(sample)) = self.nodes_.get(curr_index) {
+                // move past Ordering::Equal
+                match key.cmp(&sample.key_) {
+                    Ordering::Equal => {
+                        return Ok(curr_index); // Insert with an already existing key is a 'nop'
                     }
-                }
-            } else {
-                //println!("search down, insert after equals");
-                // we are searching down the list, stop at first Less
-                while let Some(Some(sample)) = self.nodes_.get(curr_index) {
-                    // move past Ordering::Equal
-                    if key.cmp(&sample.key_) == Ordering::Less {
+                    Ordering::Less => {
                         insert_before = Some(curr_index);
                         break;
-                    } else {
+                    }
+                    _ => {
                         curr_index = sample.next_;
                     }
                 }
             }
         } else {
-            if before_equals {
-                if cmp == Ordering::Equal {
-                    insert_before = Some(curr_index);
-                }
-                //println!("search up, curr_index={}, cmp={:?}, insert before equals insert_before:{:?}", curr_index, cmp, insert_before);
-                // we are searching up the list, stop at first Greater
-                while let Some(Some(sample)) = self.nodes_.get(curr_index) {
-                    if key.cmp(&sample.key_) == Ordering::Greater {
-                        break;
-                    } else {
-                        insert_before = Some(curr_index);
-                        curr_index = sample.prev_;
+            if cmp == Ordering::Less {
+                insert_before = Some(curr_index);
+            }
+            //println!("search up, insert after equals. tmp insert_before:{:?}", insert_before);
+            // we are searching up the list, stop at first Equal or Greater
+            while let Some(Some(sample)) = self.nodes_.get(curr_index) {
+                match key.cmp(&sample.key_) {
+                    Ordering::Equal => {
+                        return Ok(curr_index); // Insert with an already existing key is a 'nop'
                     }
-                }
-            } else {
-                if cmp == Ordering::Less {
-                    insert_before = Some(curr_index);
-                }
-                //println!("search up, insert after equals. tmp insert_before:{:?}", insert_before);
-                // we are searching up the list, stop at first Equal or Greater
-                while let Some(Some(sample)) = self.nodes_.get(curr_index) {
-                    if key.cmp(&sample.key_) != Ordering::Less {
-                        //println!("break: insert_before:{:?}", insert_before);
-                        break;
-                    } else {
+                    Ordering::Less => {
                         insert_before = Some(curr_index);
                         curr_index = sample.prev_;
                         //println!("continue: curr_index:{}", curr_index);
                     }
+                    _ => {
+                        //println!("break: insert_before:{:?}", insert_before);
+                        break;
+                    }
                 }
             }
         }
@@ -506,21 +504,14 @@ impl<'a, T: 'a, U: 'a> LinkedList<T, U>
     /// before position (i.e., either it is equivalent or goes after).
     /// If 'search_from_head' is true the search will be performed from the head otherwise from the tail.
     /// Returns None if no data is found
-    pub fn lower_bound(&self, key: T, search_from_head: bool) -> Result<Option<usize>, MapError> {
-        //println!("looking for {:?}\nin {:?}", key, self);
-        /*let equals:Vec<T> = self.iter().filter_map(|n| {if n.cmp(&key)==Ordering::Equal{Some(n.clone())} else {None}}).collect();
-        if equals.len() >= 2 {
-          println!("There were multiple alternative for lower_bound");
-          println!("{:?}", equals);
-        }*/
-
+    pub fn lower_bound(&self, key: K) -> Result<Option<usize>, MapError> {
         #[cfg(feature = "console_debug")]
         {
             let mut iter = self.iter();
             let mut flips = 0_usize;
-            let mut last_cmp = iter.next().map(|(first,_)| key.cmp(first));
+            let mut last_cmp = iter.next().map(|(first, _)| key.cmp(first));
 
-            for (node,_) in iter {
+            for (node, _) in iter {
                 let cmp = Some(key.cmp(node));
                 if cmp != last_cmp {
                     last_cmp = cmp;
@@ -529,77 +520,68 @@ impl<'a, T: 'a, U: 'a> LinkedList<T, U>
             }
             if flips > 1 {
                 println!("\nkey={:?}", key);
-                for (n,_) in self.iter() {
+                for (n, _) in self.iter() {
                     println!("key.cmp({:?})=={:?}-{:?}", n, n.cmp(&key), key.cmp(n));
                 }
-            } else {
-                //println!("{} flips found", flips);
             }
         }
 
-        if search_from_head {
-            // sequential search from the front
-            if self.head_ == OUT_OF_BOUNDS {
-                return Ok(None);
-            }
-            let mut curr_index = self.head_;
-            while let Some(Some(sample)) = self.nodes_.get(curr_index) {
-                if key.cmp(&sample.key_) != Ordering::Greater {
-                    //println!("found :{:?}, order:{:?}", sample.key,  key.cmp(&sample.key));
-                    return Ok(Some(curr_index));
-                } else {
-                    curr_index = sample.next_;
-                }
-            }
-        } else {
-            // sequential search from the rear
-            if self.tail_ == OUT_OF_BOUNDS {
-                return Ok(None);
-            }
-            let mut last_match: Option<usize> = None;
-
-            let mut curr_index = self.tail_;
-            while let Some(Some(sample)) = self.nodes_.get(curr_index) {
-                if key.cmp(&sample.key_) != Ordering::Greater {
-                    //println!("ignoring :{:?} ", sample.key);
-                    last_match = Some(curr_index);
-                    curr_index = sample.prev_;
-                } else {
-                    return if let Some(last_match) = last_match {
-                        //println!("found :{:?} ", sample.key);
-                        Ok(Some(last_match))
-                    } else {
-                        //println!("found nothing :{:?}", last_match);
-                        Ok(None)
-                    };
-                }
-            }
-            return if let Some(last_match) = last_match {
-                //println!("found index:{:?} ", last_match);
-                Ok(Some(last_match))
-            } else {
-                //println!("found nothing :{:?}", last_match);
-                Ok(None)
-            };
+        // sequential search from the rear
+        if self.tail_ == OUT_OF_BOUNDS {
+            return Ok(None);
         }
-        Ok(None)
+        let mut last_match: Option<usize> = None;
+        let mut curr_index = self.tail_;
+        while let Some(Some(sample)) = self.nodes_.get(curr_index) {
+            if key.cmp(&sample.key_) != Ordering::Greater {
+                //println!("ignoring :{:?} ", sample.key);
+                last_match = Some(curr_index);
+                curr_index = sample.prev_;
+            } else {
+                return Ok(last_match);
+            }
+        }
+        Ok(last_match)
     }
 
     #[inline(always)]
     /// Pop the head item
-    pub fn pop_front(&mut self) -> Result<Option<T>, MapError> {
+    /// ```
+    /// # use cpp_map::LinkedList;
+    /// let mut ll = LinkedList::<i8, i8>::default();
+    /// let _ = ll.ordered_insert(1, 0); // 0
+    /// let _ = ll.ordered_insert(2, 1); // 1
+    /// assert_eq!(ll.pop_front().unwrap().unwrap(), (1_i8,0_i8));
+    /// assert_eq!(ll.pop_front().unwrap().unwrap(), (2_i8,1_i8));
+    /// ```
+    pub fn pop_front(&mut self) -> Result<Option<(K, V)>, MapError> {
         self.remove_(self.head_)
     }
 
     #[inline(always)]
     /// Pop the tail item
-    pub fn pop_back(&mut self) -> Result<Option<T>, MapError> {
+    /// ```
+    /// # use cpp_map::LinkedList;
+    /// let mut ll = LinkedList::<i8, i8>::default();
+    /// let _ = ll.ordered_insert(1, 0); // 0
+    /// let _ = ll.ordered_insert(2, 1); // 1
+    /// assert_eq!(ll.pop_back().unwrap().unwrap(), (2_i8,1_i8));
+    /// assert_eq!(ll.pop_back().unwrap().unwrap(), (1_i8,0_i8));
+    /// ```
+    pub fn pop_back(&mut self) -> Result<Option<(K, V)>, MapError> {
         self.remove_(self.tail_)
     }
 
     #[inline(always)]
-    /// Pop the head item
-    pub fn peek_front_k(&self) -> Option<&T> {
+    /// Peek the head key
+    /// ```
+    /// # use cpp_map::LinkedList;
+    /// let mut ll = LinkedList::<i8, i8>::default();
+    /// let _ = ll.ordered_insert(1, 0); // 0
+    /// let _ = ll.ordered_insert(2, 1); // 1
+    /// assert_eq!(ll.peek_front_k().unwrap(), &1_i8);
+    /// ```
+    pub fn peek_front_k(&self) -> Option<&K> {
         match self.nodes_.get(self.head_) {
             Some(Some(node)) => Some(&node.key_),
             _ => None,
@@ -607,8 +589,15 @@ impl<'a, T: 'a, U: 'a> LinkedList<T, U>
     }
 
     #[inline(always)]
-    /// Pop the tail item
-    pub fn peek_back_k(&self) -> Option<&T> {
+    /// Peek the tail key
+    /// ```
+    /// # use cpp_map::LinkedList;
+    /// let mut ll = LinkedList::<i8, i8>::default();
+    /// let _ = ll.ordered_insert(1, 0); // 0
+    /// let _ = ll.ordered_insert(2, 1); // 1
+    /// assert_eq!(ll.peek_back_k().unwrap(), &2_i8);
+    /// ```
+    pub fn peek_back_k(&self) -> Option<&K> {
         match self.nodes_.get(self.tail_) {
             Some(Some(node)) => Some(&node.key_),
             _ => None,
@@ -629,17 +618,13 @@ impl<'a, T: 'a, U: 'a> LinkedList<T, U>
 
     #[inline(always)]
     /// Remove the item at index, return item value if found
-    fn remove_(&mut self, index: usize) -> Result<Option<T>, MapError> {
-        let rv = self.remove__(index, false)?;
+    fn remove_(&mut self, index: usize) -> Result<Option<(K, V)>, MapError> {
+        let rv = self.remove__(index)?;
         Ok(Some(rv.1))
     }
 
     /// Disconnect and remove the item at index, return item value if found
-    fn remove__(
-        &mut self,
-        index: usize,
-        only_disconnect: bool,
-    ) -> Result<(usize, T, usize), MapError> {
+    fn remove__(&mut self, index: usize) -> Result<(usize, (K, V), usize), MapError> {
         if self.head_ == OUT_OF_BOUNDS {
             return Err(MapError::InternalError(format!(
                 "Could not find element to remove {}:{}",
@@ -669,8 +654,6 @@ impl<'a, T: 'a, U: 'a> LinkedList<T, U>
                                 line!()
                             )));
                         }
-                    } else {
-                        // node had no next
                     }
 
                     // Check prev node
@@ -685,8 +668,6 @@ impl<'a, T: 'a, U: 'a> LinkedList<T, U>
                                 line!()
                             )));
                         }
-                    } else {
-                        // node had no prev
                     }
                     Some(operation)
                 } else {
@@ -701,7 +682,7 @@ impl<'a, T: 'a, U: 'a> LinkedList<T, U>
                 None
             };
             if let Some(operation) = operation {
-                Some(self.erase_node_(operation, only_disconnect)?)
+                Some(self.erase_node_(operation)?)
             } else {
                 None
             }
@@ -722,16 +703,15 @@ impl<'a, T: 'a, U: 'a> LinkedList<T, U>
     fn erase_node_(
         &mut self,
         operation: EraseOperation,
-        only_disconnect: bool,
-    ) -> Result<(usize, T, usize), MapError> {
+    ) -> Result<(usize, (K, V), usize), MapError> {
         //println!("erase_operation {:?}", operation);
         match (operation.change_prev_, operation.change_next_) {
             (Some((prev_i, new_next)), Some((next_i, new_prev))) => {
                 #[cfg(feature = "console_debug")]
-                    {
-                        assert_eq!(new_next, next_i);
-                        assert_eq!(prev_i, new_prev);
-                    }
+                {
+                    assert_eq!(new_next, next_i);
+                    assert_eq!(prev_i, new_prev);
+                }
                 match self.nodes_.get_mut(prev_i) {
                     Some(Some(node)) => {
                         node.next_ = new_next;
@@ -790,18 +770,16 @@ impl<'a, T: 'a, U: 'a> LinkedList<T, U>
         }
         match self.nodes_.get_mut(operation.erase_) {
             Some(old_head) => {
-                if only_disconnect {
-                    // only disconnect the node, i.e. leave it in place - disconnected.
-                    if let Some(old_head) = old_head {
-                        return Ok((old_head.prev_, old_head.key_.clone(), old_head.next_));
-                    }
-                } else {
-                    // Replace the node with None
-                    if let Some(old_head) = old_head.take() {
-                        self.id_pool_.push(operation.erase_);
-                        return Ok((old_head.prev_, old_head.key_, old_head.next_));
-                    }
+                // Replace the node with None
+                if let Some(old_head) = old_head.take() {
+                    self.id_pool_.push(operation.erase_);
+                    return Ok((
+                        old_head.prev_,
+                        (old_head.key_, old_head.value_),
+                        old_head.next_,
+                    ));
                 }
+
                 return Err(MapError::InternalError(format!(
                     "Should not happen error™ at {}:{}",
                     file!(),
@@ -822,25 +800,25 @@ impl<'a, T: 'a, U: 'a> LinkedList<T, U>
 
 #[derive(Clone, Debug)]
 /// A double ended iterator
-pub struct ListIterator<'a, T: 'a, U: 'a>
-    where
-        T: Clone + Debug,
-        U: Clone + Debug,
+pub struct ListIterator<'a, K: 'a, V: 'a>
+where
+    K: Debug,
+    V: Debug,
 {
-    list_: &'a LinkedList<T, U>,
+    list_: &'a LinkedList<K, V>,
     my_next_: usize,
 }
 
-impl<'a, T: 'a, U: 'a> std::iter::Iterator for ListIterator<'a, T, U>
-    where
-        T: Clone + Debug,
-        U: Clone + Debug,
+impl<'a, K: 'a, V: 'a> std::iter::Iterator for ListIterator<'a, K, V>
+where
+    K: Debug,
+    V: Debug,
 {
-    type Item = (&'a T, &'a U);
+    type Item = (&'a K, &'a V);
 
     #[inline]
     /// Step the iterator forward one step
-    fn next(&mut self) -> Option<(&'a T, &'a U)> {
+    fn next(&mut self) -> Option<(&'a K, &'a V)> {
         if self.my_next_ == OUT_OF_BOUNDS {
             return None;
         }
@@ -859,14 +837,14 @@ impl<'a, T: 'a, U: 'a> std::iter::Iterator for ListIterator<'a, T, U>
     }
 }
 
-impl<'a, T: 'a, U: 'a> DoubleEndedIterator for ListIterator<'a, T, U>
-    where
-        T: Clone + Debug,
-        U: Clone + Debug,
+impl<'a, K: 'a, V: 'a> DoubleEndedIterator for ListIterator<'a, K, V>
+where
+    K: Debug,
+    V: Debug,
 {
     #[inline]
     /// Step the iterator backward one step
-    fn next_back(&mut self) -> Option<(&'a T, &'a U)> {
+    fn next_back(&mut self) -> Option<(&'a K, &'a V)> {
         if let Some(node) = self.list_.nodes_.get(self.my_next_)? {
             if self.my_next_ == self.list_.tail_ {
                 self.my_next_ = OUT_OF_BOUNDS;
@@ -884,99 +862,45 @@ impl<'a, T: 'a, U: 'a> DoubleEndedIterator for ListIterator<'a, T, U>
 /// An effort to emulate a C++ std::map iterator in Rust.
 /// It will have functionality like:
 /// prev(), next(), get(), erase(), lower_bound(), replace_key()
-pub struct PIterator<T, U>
-    where
-        T: Clone + Debug,
-        U: Clone + Debug,
+pub struct PIterator<K, V>
+where
+    K: Debug,
+    V: Debug,
 {
     current: usize,
-    list: Rc<RefCell<LinkedList<T, U>>>,
+    list: Rc<RefCell<LinkedList<K, V>>>,
 }
 
 #[allow(dead_code)]
-impl<T, U> PIterator<T, U>
-    where
-        T: Clone + Debug + Unpin + Ord + PartialOrd,
-        U: Clone + Debug + Unpin,
+impl<K, V> PIterator<K, V>
+where
+    K: Clone + Debug + Unpin + Ord + PartialOrd,
+    V: Clone + Debug + Unpin,
 {
     /// Initiates the pointer with a list, set current to the head of the list.
-    pub fn new(list: Rc<RefCell<LinkedList<T, U>>>) -> Self {
-        let head = list.borrow().head_;
-        Self {
+    pub fn new(list: Rc<RefCell<LinkedList<K, V>>>) -> Result<Self, MapError> {
+        let head = list.try_borrow()?.head_;
+        Ok(Self {
             current: head,
             list,
-        }
+        })
     }
 
     /// Initiates the pointer with a list, set index.
-    pub fn new_2(list: Rc<RefCell<LinkedList<T, U>>>, current: usize) -> Self {
+    pub fn new_2(list: Rc<RefCell<LinkedList<K, V>>>, current: usize) -> Self {
         Self { current, list }
     }
 
     #[inline(always)]
-    /// Returns a clone of the data at current position
-    pub fn get_k(&self) -> Result<T, MapError> {
-        if self.current == OUT_OF_BOUNDS {
-            //panic!();
-            return Err(MapError::InternalError(format!(
-                "Invalid pointer (moved past start/end). {}:{}",
-                file!(),
-                line!()
-            )));
-        }
-        let node = self
-            .list
-            .borrow()
-            .nodes_
-            .get(self.current)
-            .ok_or_else(|| {
-                MapError::InternalError(format!(
-                    "Node {} not found. {}:{}",
-                    self.current,
-                    file!(),
-                    line!()
-                ))
-            })?
-            .as_ref()
-            .ok_or_else(|| {
-                MapError::InternalError(format!(
-                    "Node {} was None. {}:{}",
-                    self.current,
-                    file!(),
-                    line!()
-                ))
-            })?
-            .clone();
-        Ok(node.key_)
+    /// Returns a clone of the key at current position
+    pub fn get_k(&self) -> Result<K, MapError> {
+        Ok(self.list.try_borrow()?.get(self.current)?.0.clone())
     }
 
     #[inline(always)]
-    /// Returns a clone of the data at current position
-    pub fn get_v(&self) -> Result<U, MapError> {
-        let node = self
-            .list
-            .borrow()
-            .nodes_
-            .get(self.current)
-            .ok_or_else(|| {
-                MapError::InternalError(format!(
-                    "Node {} not found. {}:{}",
-                    self.current,
-                    file!(),
-                    line!()
-                ))
-            })?
-            .as_ref()
-            .ok_or_else(|| {
-                MapError::InternalError(format!(
-                    "Node {} was None. {}:{}",
-                    self.current,
-                    file!(),
-                    line!()
-                ))
-            })?
-            .clone();
-        Ok(node.value_)
+    /// Returns a clone of the value at current position
+    pub fn get_v(&self) -> Result<V, MapError> {
+        Ok(self.list.try_borrow()?.get(self.current)?.1.clone())
     }
 
     #[allow(clippy::should_implement_trait)]
@@ -984,21 +908,17 @@ impl<T, U> PIterator<T, U>
     /// Move to the next element.
     /// Note that this is NOT a Rust iterator next() method.
     pub fn next(&mut self) -> Result<(), MapError> {
-        let list_borrow = self.list.borrow();
+        let list_borrow = self.list.try_borrow()?;
         match list_borrow.nodes_.get(self.current) {
             Some(Some(node)) => self.current = node.next_,
             // Some(None) nodes should be inaccessible
             Some(None) => {
-                eprintln!("next() failed at index:{}", self.current);
-                for (i, n) in list_borrow.nodes_.iter().enumerate() {
-                    eprintln!(" #{}, {:?}", i, n);
-                }
                 return Err(MapError::InternalError(format!(
                     "next() failed at index:{}. {}:{}",
                     self.current,
                     file!(),
                     line!()
-                )))
+                )));
             }
             None => self.current = OUT_OF_BOUNDS,
         }
@@ -1008,15 +928,11 @@ impl<T, U> PIterator<T, U>
     #[inline(always)]
     /// Move to the previous element
     pub fn prev(&mut self) -> Result<(), MapError> {
-        let list_borrow = self.list.borrow();
+        let list_borrow = self.list.try_borrow()?;
         match list_borrow.nodes_.get(self.current) {
             Some(Some(node)) => self.current = node.prev_,
             // Some(None) nodes should be inaccessible
             Some(None) => {
-                eprintln!("prev() failed at index:{}", self.current);
-                for (i, n) in list_borrow.nodes_.iter().enumerate() {
-                    eprintln!(" #{}, {:?}", i, n);
-                }
                 return Err(MapError::InternalError(format!(
                     "prev() failed at index:{}. {}:{}",
                     self.current,
@@ -1031,48 +947,54 @@ impl<T, U> PIterator<T, U>
 
     #[inline(always)]
     /// Move to the first element
-    pub fn move_to_head(&mut self) {
-        self.current = self.list.borrow().head_
+    pub fn move_to_head(&mut self) -> Result<(), MapError> {
+        self.current = self.list.try_borrow()?.head_;
+        Ok(())
     }
 
     #[inline(always)]
     /// Move to the last element
-    pub fn move_to_tail(&mut self) {
-        self.current = self.list.borrow().tail_
+    pub fn move_to_tail(&mut self) -> Result<(), MapError> {
+        self.current = self.list.try_borrow()?.tail_;
+        Ok(())
     }
 
     #[inline(always)]
     /// Return true if pointer has *NOT* moved past beginning or end of the list
-    pub fn is_ok(&self) -> bool {
-        self.current != OUT_OF_BOUNDS
-            && matches!(self.list.borrow().nodes_.get(self.current), Some(Some(_)))
+    pub fn is_ok(&self) -> Result<bool, MapError> {
+        Ok(self.current != OUT_OF_BOUNDS
+            && matches!(
+                self.list.try_borrow()?.nodes_.get(self.current),
+                Some(Some(_))
+            ))
     }
 
     #[inline(always)]
-    /// Return true if pointer is at head position
-    pub fn is_at_head(&self) -> bool {
-        self.current == self.list.borrow().head_
+    /// Return true if pointer is at head position or if the list is empty
+    pub fn is_at_head(&self) -> Result<bool, MapError> {
+        Ok(self.current == self.list.try_borrow()?.head_)
     }
 
     #[inline(always)]
-    /// Return true if pointer is at tail position
-    pub fn is_at_tail(&self) -> bool {
-        self.current == self.list.borrow().tail_
+    /// Return true if pointer is at tail position or if the list is empty
+    pub fn is_at_tail(&self) -> Result<bool, MapError> {
+        Ok(self.current == self.list.try_borrow()?.tail_)
     }
 
     #[inline(always)]
     /// Replace current key. This will destroy the internal order of element if you
     /// replace an element with something out of order.
-    pub fn replace_key(&mut self, key: T) {
-        let mut list = std::pin::Pin::new(self.list.borrow_mut());
+    pub fn replace_key(&mut self, key: K) -> Result<(), MapError> {
+        let mut list = std::pin::Pin::new(self.list.try_borrow_mut()?);
         if let Some(Some(ref mut node)) = list.nodes_.get_mut(self.current) {
             node.key_ = key;
         }
+        Ok(())
     }
 
     #[inline(always)]
     /// returns current index
-    pub fn get_index(&self) -> usize {
+    pub fn current(&self) -> usize {
         self.current
     }
 
@@ -1080,11 +1002,8 @@ impl<T, U> PIterator<T, U>
     /// Remove the current element and return it. Move current to the old prev value if exist.
     /// Else pick old next index.
     /// Note: make sure that there are no other Pointer objects at this position.
-    pub fn remove_current(&mut self, only_disconnect: bool) -> Result<T, MapError> {
-        let rv = self
-            .list
-            .borrow_mut()
-            .remove__(self.current, only_disconnect)?;
+    pub fn remove_current(&mut self) -> Result<(K, V), MapError> {
+        let rv = self.list.try_borrow_mut()?.remove__(self.current)?;
         if rv.0 != OUT_OF_BOUNDS {
             self.current = rv.0;
         } else {
@@ -1098,8 +1017,8 @@ impl<T, U> PIterator<T, U>
     /// Lower bound item is the first element in the container whose key is not considered to go
     /// before position (i.e., either it is equivalent or goes after).
     /// Returns a Pointer where is_ok() returns false if no data is found
-    pub fn lower_bound(list:Rc<RefCell<LinkedList<T, U>>>, key: T, from_head: bool) -> Result<Self, MapError> {
-        let position = list.borrow().lower_bound(key, from_head)?;
+    pub fn lower_bound(list: Rc<RefCell<LinkedList<K, V>>>, key: K) -> Result<Self, MapError> {
+        let position = list.try_borrow()?.lower_bound(key)?;
         if let Some(position) = position {
             Ok(Self {
                 list,
@@ -1115,20 +1034,20 @@ impl<T, U> PIterator<T, U>
     }
 }
 
-impl<T, U> Debug for PIterator<T, U>
-    where
-        T: Clone + Debug + Unpin + Ord + PartialOrd,
-        U: Clone + Debug + Unpin,
+impl<K, V> Debug for PIterator<K, V>
+where
+    K: Debug + Unpin + Ord + PartialOrd,
+    V: Debug + Unpin,
 {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         writeln!(f, "PIterator({})", self.current)
     }
 }
 
-impl<T, U> Clone for PIterator<T, U>
-    where
-        T: Clone + Debug + Unpin + Ord + PartialOrd,
-        U: Clone + Debug + Unpin,
+impl<K, V> Clone for PIterator<K, V>
+where
+    K: Debug + Unpin + Ord + PartialOrd,
+    V: Debug + Unpin,
 {
     fn clone(&self) -> Self {
         Self {
